package externaljava;


import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;

import com.sun.corba.se.impl.protocol.giopmsgheaders.Message;
import com.sun.jdi.connect.Transport;
import com.sun.xml.internal.messaging.saaj.packaging.mime.MessagingException;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.MimeBodyPart;
import com.sun.xml.internal.messaging.saaj.packaging.mime.internet.MimeMultipart;

public class MailSender {

    //To use methods without smtp params you must preconfigure these fields. At least SMTP_HOST_VALUE, if sendmail/postfix is placed on the same host where app is
    public static final String SMTP_HOST_VALUE = "";
    public static final String SMTP_USER_VALUE = "";
    public static final String SMTP_PASS_VALUE = "";
    private static final Logger _logger = Logger.getLogger(MailSender.class);

    /**
     * @param smtpHost         SMTP host, mandatory
     * @param smtpUser         SMTP user, if no user specified (null or empty string), mail server auth will be disabled.
     * @param smtpPass         SMTP password
     * @param to               array of receivers (any malformed addresses will be skipped)
     * @param cc               array of cc-receivers (any malformed addresses will be skipped)
     * @param bcc              array of bcc-receivers (any malformed addresses will be skipped)
     * @param from             field 'from'
     * @param subject          mail subject
     * @param content          mail text content
     * @param filePath         absolute path to attachment (not a BIRT report!)
     * @param fileName         name, that will be used for attachment
     * @param birtHost         BIRT hostname
     * @param birtReport       BIRT report file full path
     * @param birtParams       params for BIRT report
     * @param reportAttachName name that will be used for report's attachment
     * @param function         execution type
     * @return "0" if success,'1' if errors occurred
     */
    public static String sendMail(String smtpHost, String smtpUser, String smtpPass, String[] to, String[] cc, String[] bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, Map<String, String> birtParams, String reportAttachName, String function) {
        Session sess = createSession(smtpHost, smtpUser, smtpPass);
        return sendMail(sess, BirtReportHelper.createListFromArray(to), BirtReportHelper.createListFromArray(cc), BirtReportHelper.createListFromArray(bcc), from, subject, content, filePath, fileName, birtHost, birtReport, birtParams, reportAttachName, function);
    }

    public static String sendMail(String smtpHost, String smtpUser, String smtpPass, String[] to, String[] cc, String[] bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, String[] birtParams, String reportAttachName, String function) {
        return sendMail(smtpHost, smtpUser, smtpPass, to, cc, bcc, from, subject, content, filePath, fileName, birtHost, birtReport, BirtReportHelper.createMapFromArray(birtParams), reportAttachName, function);
    }

    public static String sendMailA(String smtpHost, String smtpUser, String smtpPass, List<String> to, List<String> cc, List<String> bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, List<String> birtParams, String reportAttachName, String function) {
        Session sess = createSession(smtpHost, smtpUser, smtpPass);
        return sendMail(sess, to, cc, bcc, from, subject, content, filePath, fileName, birtHost, birtReport, BirtReportHelper.createMapFromList(birtParams), reportAttachName, function);
    }


    /**
     * Method can be used instead of upper one - it supports multiple attachments, but BIRT report must be in 'attachments map',if it's required
     *
     * @param smtpHost    SMTP host, mandatory
     * @param smtpUser    SMTP user, if no user specified (null or empty string), mail server auth will be disabled.
     * @param smtpPass    SMTP password
     * @param to          array of receivers (any malformed addresses will be skipped)
     * @param cc          array of cc-receivers (any malformed addresses will be skipped)
     * @param bcc         array of bcc-receivers (any malformed addresses will be skipped)
     * @param from        field 'from'
     * @param subject     mail subject
     * @param content     mail text content
     * @param attachments Map<String,String> that contains pairs of 'attachment name (in mail message)'-'attachment file full path' values. if any of it is null attachemnt won't be used.
     * @return "0" if everything was fine, "1" if some error occurred
     */
    public static String sendMail(String smtpHost, String smtpUser, String smtpPass, String[] to, String[]
            cc, String[] bcc, String from, String subject, String content, Map<String, String> attachments) {
        Session instance = createSession(smtpHost, smtpUser, smtpUser);
        return sendMail(instance, BirtReportHelper.createListFromArray(to), BirtReportHelper.createListFromArray(cc), BirtReportHelper.createListFromArray(bcc), from, subject, content, attachments);
    }

    public static String sendMail(String smtpHost, String smtpUser, String smtpPass, String[] to, String[] cc, String[] bcc, String from, String subject, String content, String[] attachments) {
        return sendMail(smtpHost, smtpUser, smtpPass, to, cc, bcc, from, subject, content, BirtReportHelper.createMapFromArray(attachments));
    }

    public static String sendMailB(String smtpHost, String smtpUser, String smtpPass, List<String> to, List<String>
            cc, List<String> bcc, String from, String subject, String content, List<String> attachments) {
        Session sess = createSession(smtpHost, smtpUser, smtpPass);
        return sendMail(sess, to, cc, bcc, from, subject, content, BirtReportHelper.createMapFromList(attachments));
    }


    /**
     * Same as method, that receives SMTP credentials, but this one uses preconfigured session (from static constants)
     */
    public static String sendMail(String[] to, String[] cc, String[] bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, Map<String, String> birtParams, String reportAttachName, String function) {
        return sendMail(getSession(), BirtReportHelper.createListFromArray(to), BirtReportHelper.createListFromArray(cc), BirtReportHelper.createListFromArray(bcc), from, subject, content, filePath, fileName, birtHost, birtReport, birtParams, reportAttachName, function);
    }

    public static String sendMail(String[] to, String[] cc, String[] bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, String[] birtParams, String reportAttachName, String function) {
        return sendMail(to, cc, bcc, from, subject, content, filePath, fileName, birtHost, birtReport, BirtReportHelper.createMapFromArray(birtParams), reportAttachName, function);
    }

    public static String sendMailC(List<String> to, List<String> cc, List<String> bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, List<String> birtParams, String reportAttachName, String function) {
        return sendMail(getSession(), to, cc, bcc, from, subject, content, filePath, fileName, birtHost, birtReport, BirtReportHelper.createMapFromList(birtParams), reportAttachName, function);
    }


    /**
     * Same as  method, that receives SMTP credentials, but this one uses preconfigured session (from static constants)
     */
    public static String sendMail(String[] to, String[] cc, String[] bcc, String from, String subject, String content, Map<String, String> attachments) {
        return sendMail(getSession(), BirtReportHelper.createListFromArray(to), BirtReportHelper.createListFromArray(cc), BirtReportHelper.createListFromArray(bcc), from, subject, content, attachments);
    }

    public static String sendMail(String[] to, String[] cc, String[] bcc, String from, String subject, String content, String[] attachments) {
        return sendMail(to, cc, bcc, from, subject, content, BirtReportHelper.createMapFromArray(attachments));
    }

    public static String sendMailD(List<String> to, List<String> cc, List<String> bcc, String from, String subject, String content, List<String> attachments) {
        return sendMail(getSession(), to, cc, bcc, from, subject, content, BirtReportHelper.createMapFromList(attachments));
    }


    private static String sendMail(Session session, List<String> to, List<String> cc, List<String> bcc, String from, String subject, String content, String filePath, String fileName, String birtHost, String birtReport, Map<String, String> birtParams, String reportAttachName, String function) {
        File report = null;
        _logger.warn("[MS] Mail function invoked. Selected function:" + function);
        if (PDF_ONLY.equalsIgnoreCase(function) || EMAIL_AND_ATTACH.equalsIgnoreCase(function)) {
            createBirtLog(birtHost, birtReport, birtParams, reportAttachName, function, filePath, fileName);
            report = BirtReportHelper.getBirtReportFile(birtHost, birtReport, birtParams, PDF_ONLY.equalsIgnoreCase(function));
            if (report == null) {
                _logger.error("[MS] Report creation was started,but report is null!");
                return ERROR;
            }
            _logger.warn("[MS] Report successfully generated and received.");
        }
        if (EMAIL_AND_ATTACH.equalsIgnoreCase(function) || EMAIL_ONLY.equalsIgnoreCase(function)) {
            Map<String, String> attachments = new HashMap<String, String>();
            if (BirtReportHelper.isNotEmpty(fileName) && BirtReportHelper.isNotEmpty(filePath)) {
                attachments.put(fileName, filePath);
            }
            if (report != null) {
                attachments.put(reportAttachName, report.getAbsolutePath());
            }
            return sendMail(session, to, cc, bcc, from, subject, content, attachments);
        }

        return SUCCESS;
    }

    private static String sendMail(Session session, List<String> to, List<String> cc, List<String> bcc, String from, String subject, String content, Map<String, String> attachments) {
        try {
            createSendMailLog(session, to, cc, bcc, from, subject, attachments);
            Message message = prepareMessage(session, subject, from, to, bcc, cc);


            message.setContent(createMessageContent(attachments, content));
            message.saveChanges();
            _logger.debug("[MS] Trying to initialize mailing transport....");
            Transport transport = session.getTransport(PROTOCOL);
            transport.connect(session.getProperty(SMTP_USER), session.getProperty(SMTP_PASS));
            transport.sendMessage(message, message.getAllRecipients());
            transport.close();
            _logger.debug("[MS] Message successfully sent!");
        } catch (MessagingException e) {
            _logger.error("[MS] Failed to send message! Cause: " + e.getMessage());
            return ERROR;
        }
        return SUCCESS;
    }


    private static Properties createProperties(String smtpHost, String smtpUser, String smtpPass) {
        Properties properties = new Properties();
        properties.put(SMTP_HOST, smtpHost);
        if (BirtReportHelper.isEmpty(smtpHost)) {
            _logger.warn("[MS] SMTP_HOST not initialized. Using 'localhost'...");
            properties.put(SMTP_HOST, "localhost");
        }
        properties.put("mail.debug", DEBUG_MAIL);
        if (BirtReportHelper.isNotEmpty(smtpUser)) {
            properties.put(SMTP_AUTH, Boolean.TRUE.toString());
            properties.put(SMTP_USER, smtpUser);
            properties.put(SMTP_PASS, smtpPass);
        }
        return properties;
    }

    private static Message prepareMessage(Session session, String subj, String from, List<String> to, List<String> bcc, List<String> cc) throws MessagingException {
        Message message = new MimeMessage(session);
        message.setSubject(subj == null ? "" : subj);
        message.setSentDate(new Date());
        message.setFrom(new InternetAddress(from));
        handleRecipients(Message.RecipientType.TO, to, message);
        handleRecipients(Message.RecipientType.BCC, bcc, message);
        handleRecipients(Message.RecipientType.CC, cc, message);
        _logger.debug("[MS] Base Mail Message created!");
        return message;

    }

    private static Multipart createMessageContent(Map<String, String> attachments, String content) throws MessagingException {
        _logger.debug("[MS] Appending message text...");
        Multipart multipart = new MimeMultipart();

        MimeBodyPart text = new MimeBodyPart();
        text.setContent(content, TEXT_TYPE);
        multipart.addBodyPart(text);
        if (BirtReportHelper.isNotEmpty(attachments)) {
            _logger.debug("[MS] Attachments found! Appending...");
            for (Map.Entry<String, String> entry : attachments.entrySet()) {
                if (BirtReportHelper.isNotEmpty(entry.getKey()) && BirtReportHelper.isNotEmpty(entry.getValue())) {
                    MimeBodyPart attach = new MimeBodyPart();

                    attach.setFileName(entry.getKey());
                    attach.setDataHandler(new DataHandler(new FileDataSource(entry.getValue())));
                    multipart.addBodyPart(attach);
                }
            }
        }
        _logger.debug("[MS] Full message created!");
        return multipart;
    }

    private static void handleRecipients(Message.RecipientType type, List<String> rec, Message mimeMessage) {
        if (BirtReportHelper.isNotEmpty(rec)) {
            for (String s : rec) {
                try {
                    if (BirtReportHelper.isNotEmpty(s)) {
                        mimeMessage.addRecipient(type, new InternetAddress(s));
                    }
                } catch (MessagingException e) {
                    _logger.warn("[MS] Broken address passed <" + s + ">.");
                }
            }
        }
    }

    private static Session getSession() {
        if (session == null) {
            session = Session.getInstance(createProperties(SMTP_HOST_VALUE, SMTP_USER_VALUE, SMTP_PASS_VALUE), null);
            _logger.warn("[MS] Initializing static session");
        }
        return session;
    }

    private static Session createSession(String smtpHost, String smtpUser, String smtpPass) {
        return Session.getInstance(createProperties(smtpHost, smtpUser, smtpPass), null);
    }

    private static void createBirtLog(String birtHost, String birtReport, Map<String, String> params, String attachName, String fnc, String filePath, String fileName) {
        StringBuilder builder = new StringBuilder();
        builder.append("[MS] BIRT call invoked. Data:\n")
                .append("HOST: ").append(birtHost)
                .append("\nREPORT:").append(birtReport)
                .append("\nPARAMS:{").append(BirtReportHelper.join(params, null)).append("}")
                .append("\nREPORT_ATTACH_NAME: ").append(attachName)
                .append("\nEXTRA_ATTACH: {NAME:").append(fileName).append("\tPATH:").append(filePath).append("}\nFUNCTION:").append(fnc);

        _logger.warn(builder.toString());
    }

    private static void createSendMailLog(Session session, List<String> to, List<String> cc, List<String> bcc, String from, String subj, Map<String, String> attach) {
        StringBuilder builder = new StringBuilder();
        builder.append("[MS] Mail sending invoked. Params:\n")
                .append("Session: {host:").append(session.getProperty(SMTP_HOST)).append(",user:").append(session.getProperty(SMTP_USER)).append(",pass:").append(session.getProperty(SMTP_PASS)).append("}\n")
                .append("Targets: {TO: {").append(BirtReportHelper.join(to, null)).append("}, CC:{").append(BirtReportHelper.join(cc, null)).append("},BCC:{").append(BirtReportHelper.join(bcc, null)).append("}}\n")
                .append("FROM: ").append(from).append(",SUBJ:").append(subj).append("\n")
                .append("ATTACH: {").append(BirtReportHelper.join(attach, null)).append("}\n");
        _logger.warn(builder.toString());
    }


    public static final String EMAIL_ONLY = "E";
    public static final String PDF_ONLY = "C";
    public static final String EMAIL_AND_ATTACH = "B";

    public static final String ERROR = "1";
    public static final String SUCCESS = "0";

    private static final String SMTP_HOST = "mail.smtp.host";
    private static final String SMTP_AUTH = "mail.smtp.auth";
    private static final String SMTP_USER = "mail.smtp.user";
    private static final String SMTP_PASS = "mail.smtp.password";
    private static final String TEXT_TYPE = "text/plain";
    private static final String PROTOCOL = "smtp";
    private static final boolean DEBUG_MAIL = true;

    private static Session session;

}